/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: PixelView Play TV Pro 878+FM remote control driver.
 Valid for:           

 Prototype (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>
 Copyright (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// Key codes from: Oleg Kostiuchenko, <oleg@ps.kiev.ua>

// BTTool.exe display for "PixelView Play TV Pro 878+FM":
// GPIO_Dma_Ctl - 00000004; (0000c007 when TV window open)
// GPIO_Out_En  - 00000000; (000000ff when TV window open)
// GPIO_Data    - remarked column (for pressed - released
//                keys and loaded/unloaded JTV).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 00ffxxxx (when JTV-window is open)
// GPIO_Data    - 00ffffff (when JTV-window is closed)

// zero-bit for below keys (values are b and 8) can actualy differs

KeyDef keys[]=
{                                                               
															//   x=[b,8]
//                                         JTV is loaded        JTV is NOT loaded
// 1-st row keys layout                   Pressed    Released   Pressed    Released
        { 0x43, "Key_TV/FM", "" },	//00ff43e1 - 00ffc3e1 / 00ff43ex - 00ffc3ex
        { 0x47, "Key_Autoscan", "" },	//00ff47e1 - 00ffc7e1 / 00ff47ex - 00ffc7ex
        { 0x5c, "Key_Zoom", "" },	//00ff5ce1 - 00ffdce1 / 00ff5cex - 00ffdcex
        { 0x5e, "Key_Power", "Power" },	//00ff5ee1 - 00ffdee1 / 00ff5eex - 00ffdeex

// 2-nd keys row
        { 0x41, "Key_1", "1" },		//00ff41e1 - 00ffc1e1 / 00ff41ex - 00ffc1ex
        { 0x4b, "Key_2", "2" },		//00ff4be1 - 00ffcbe1 / 00ff4bex - 00ffcbex
        { 0x5b, "Key_3", "3" },		//00ff5be1 - 00ffdbe1 / 00ff5bex - 00ffdbex
        { 0x5f, "Key_VolumeUp", "VolumeUp"},	//00ff5fe1 -00ffdfe1 / 00ff5fex -00ffdfex

//3-rd row keys
        { 0x45, "Key_4", "4" },		//00ff45e1 - 00ffc5e1 / 00ff45ex - 00ffc5ex
        { 0x49, "Key_5", "5" },		//00ff49e1 - 00ffc9e1 / 00ff49ex - 00ffc9ex
        { 0x55, "Key_6", "6" },		//00ff55e1 - 00ffd5e1 / 00ff55ex - 00ffd5ex
        { 0x57, "Key_VolumeDown", "VolumeDown" }, //00ff57e1 - 00ffd7e1 / 00ff57ex - 00ffd7ex

//4-th row keys
        { 0x46, "Key_7", "7" },		//00ff46e1 - 00ffc6e1 / 00ff46ex - 00ffc6ex
        { 0x4a, "Key_8", "8" },		//00ff4ae1 - 00ffcae1 / 00ff4aex - 00ffcae8
        { 0x52, "Key_9", "9" },		//00ff52e1 - 00ffd2e1 / 00ff52ex - 00ffd2e8
        { 0x56, "Key_ChannelUp", "ChannelUp" },	//00ff56e1 - 00ffd6e1 / 00ff56ex - 00ffd6ex

//5-th row keys
        { 0x42, "Key_0", "0" },		//00ff42e1 - 00ffc2e1 / 00ff42ex - 00ffc2ex             
        { 0x53, "Key_Loop", "Recall" },	//00ff53e1 - 00ffd3e1 / 00ff53ex - 00ffd3ex             
        { 0x50, "Key_Enter", "Enter" },	//00ff50e1 - 00ffd0e1 / 00ff50ex - 00ffd0ex             
        { 0x54, "Key_ChannelDown", "ChannelDown", }, //00ff54e1 - 00ffd4e1 / 00ff54ex - 00ffd4ex

//6-th row keys
        { 0x40, "Key_Source", "" },	//00ff40e1 - 00ffc0e1 / 00ff40ex - 00ffc0ex
        //no key                                                                   
        //no key                                                                   
        { 0x58, "Key_Mute", "" },	//00ff58e1 - 00ffd8e1 / 00ff58ex - 00ffd8ex

//7-th row keys
        { 0x44, "Key_Left", "Left" },	//00ff44e1 - 00ffc4e1 / 00ff44ex - 00ffc4ex 
        { 0x4e, "Key_PlusMinus", "" },   //00ff4ee1 - 00ffcee1 / 00ff4eex - 00ffceex
        //no key                                                                   
        { 0x4c, "Key_Right", "Right" },	//00ff4ce1 - 00ffcce1 / 00ff4cex - 00ffccex 

//8-th row keys
        { 0x4d, "Key_Reset", ""},	//00ff4de1 - 00ffcde1 / 00ff4dex - 00ffcdex
        //no key                                                                   
        //no key                                                                   
        { 0x4f, "Key_Freeze", "" },	//00ff4fe1 - 00ffcfe1 / 00ff4fex - 00ffcfex
};
 
// Please, mark with ">>" sign correct answer below or add your own.
// When any key is pressed, GPIO_Data value:
// >> 1. Strongly keeps the value
//    2. In random intervals (=> ??? ms) changes to value - ??????
//    3. Other case ??


typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=1000;	// default first delay for a key
INT repeat_delay=100;	// default repeat delay for a key


void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
                log("> Initializing PixelView Play TV Pro 878+FM remote module.    \n");
                log("> Copyright (c) 2000 by Ivan Kolin, <ivkolin@mail.ru>.\n");
				log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
		};



        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();

                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // initial repeat delay
					DWORD press_flag=0;
					DWORD test_code=0;

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0xc007);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0xff));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0xf4e700);
                   
				   
                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
						 
                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
 						 
                         value = ((gpio_data &0x0000ff00)>>8);
                         press_flag = ((gpio_data &0x0000f000)>>12);
						 
						 // --- Test block begin - comment all while build checked out driver
						 //if (test_code != value)
						 //{
						 //	log(">> ---- Test information ---- \n");
						 //	log(">>      Tick_Counter: %d\n",GetTickCount());
						 //	log(">>         GPIO_DATA: %x\n",gpio_data);
						 //	log(">>  Current_Key_Code: %x\n",value);
						 //	log(">> Previous_Key_Code: %x\n",last_code);
						 //	log(">>    Press_Key_Flag: %x\n",press_flag);
						 //	test_code = value;
						 //}
           				 // --- Test block end


                         if ((press_flag == 0x4) | (press_flag == 0x5)) // any key is pressed
                         {	 
                             if (last_code == value)
                             {     // auto repeat for any pressed key
                                   //log(">>    Press_Key_Auto: %x\n",value); //for test
                            	   AddStroke(last_code);
                            	   key_pause = repeat_delay;
                             } 
                             else
                             {	   // first run for any pressed key 
                                   //log(">>   Press_Key_First: %x\n",value); //for test
                            	   last_code = value;
                                   AddStroke(last_code);
                                   key_pause = first_delay;
                             }
                         }
                         else  // no one key is pressed
                         {	  
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
                             }
							   
							   
                                key_pause = 20;
                         };
				
                    };

                log("> Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};
